"""DO NOT MODIFY. Auto-generated by script/fingerprint_frontend."""

FINGERPRINTS = {
    "core.js": "457d5acd123e7dc38947c07984b3a5e8",
    "frontend.html": "829ee7cb591b8a63d7f22948a7aeb07a",
    "mdi.html": "b399b5d3798f5b68b0a4fbaae3432d48",
    "panels/ha-panel-dev-event.html": "3cc881ae8026c0fba5aa67d334a3ab2b",
    "panels/ha-panel-dev-info.html": "34e2df1af32e60fffcafe7e008a92169",
    "panels/ha-panel-dev-service.html": "bb5c587ada694e0fd42ceaaedd6fe6aa",
    "panels/ha-panel-dev-state.html": "4608326978256644c42b13940c028e0a",
    "panels/ha-panel-dev-template.html": "0a099d4589636ed3038a3e9f020468a7",
    "panels/ha-panel-history.html": "efe1bcdd7733b09e55f4f965d171c295",
    "panels/ha-panel-iframe.html": "d920f0aa3c903680f2f8795e2255daab",
    "panels/ha-panel-logbook.html": "66108d82763359a218c9695f0553de40",
    "panels/ha-panel-map.html": "af7d04aff7dd5479c5a0016bc8d4dd7d"
}
